 /* 4-1-03
The purpose of these functions is to compute the likelihood of non-decomposable
graphs by combining the likelihood calculation for cliques and separators, and
the Importance sampling algorithm for prime components that are not complete.
This function will take a list of the prime components, determine which are 
complete, execute the appropriate computation, and then recombine them. 
*/
using namespace std;

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <limits.h>
#include <iomanip>
#include <iostream>
#include <time.h>
#include "graph.h"
#include "various.h"
#include "ISampling.h"

#define PI 3.141592653589793

extern "C" {
void dpotrf_(char *, int *, double *, int *, int *);
void dgetrf_(int *, int *, double *, int *, int *, int *);
void dgetri_(int *, double *, int *, int *, double *, int *, int *);

}

/*this says cliques, but it can be used for separators too*/

double cliquelike(int *vertices, int nvertices, double ** suffdat, double tau, double delta){
  extern double **Data;
  extern int SampleSize;
  extern int NumberOfGenes;

  double * Phi_star;
  char UPLO[2]="L";
  int INFO;
  
  double answer;
  int i,j;

   Phi_star=new double[nvertices*nvertices];
  
  answer=0;

  if(nvertices >1){
   /*gamma part of both prior and posterior*/
  for(i=0; i< nvertices; i++){
    answer=answer-lgamma((delta+((double) nvertices-i)-1.0)/2.0);
    answer=answer+lgamma((delta+((double) SampleSize+nvertices-i)-1.0)/2.0);
    }
  /*Determinant of Phi */
  answer=answer+log(tau)*((double) nvertices)*(delta+nvertices-1.0)/2.0;

      
      int index=0;
  for(i=0; i< nvertices; i++){
    for(j=0; j< nvertices; j++){
      Phi_star[index]=suffdat[vertices[i]][vertices[j]];
      if(i==j) Phi_star[index]=Phi_star[index]+tau;

      
      index++;
   }
    
  }

  

  dpotrf_(UPLO, &nvertices, Phi_star, &nvertices, &INFO);
  /*determinant of Phi_Star*/
  
  for(i=0; i< nvertices; i++){
    index=i*nvertices+i;
    answer=answer-log(Phi_star[index])*(delta+((double) nvertices+SampleSize)-1.0);
   }
  }

  else{
     answer += lgamma(((double)(delta+SampleSize))/2.0);
    answer -= lgamma(delta/2.0);
    answer += delta*log(tau)/2.0;
    answer -= ((double)(delta+SampleSize))*log(tau+suffdat[vertices[0]][vertices[0]])/2.0;
  }


 
    delete[] Phi_star;


  return answer;
   
}



int complete(int ** incidence, int * vertices, int nvertices){
int i,j, answer;
answer=1;
 if(nvertices >1){
   i=0;	
   while(answer==1 && i<nvertices){
     j=i+1;
     while(answer==1 && j<nvertices){
       if(incidence[vertices[i]][vertices[j]]!=1) answer=0;
       j++;}
     i++;
   }}

return answer;
}

double likecomb(Graph *graph, double ** suffdat, double tau, double delta){
  extern double **Data;
  extern int SampleSize;
  extern int NumberOfGenes;
int i,j;
double answer, posNC, priNC;

answer=-((double)SampleSize)*((double)NumberOfGenes)*log(PI)/2.0;
  for(i=0; i<graph->nCliques; i++){
 
   if(graph->CliquesDimens[i]< SampleSize){
     if(complete(graph->Edge, graph->Cliques[i], graph->CliquesDimens[i])){

       posNC=cliquelike(graph->Cliques[i], graph->CliquesDimens[i], suffdat, tau, 
	   delta);
       answer=answer+posNC;

       

     }
     else{
       int k, nsamples;
       priNC=0;
       if(graph->CliquesDimens[i]*graph->CliquesDimens[i]*graph->CliquesDimens[i]/2 >1000)
	 nsamples=graph->CliquesDimens[i]*graph->CliquesDimens[i]*graph->CliquesDimens[i];
       else nsamples=2000;
             
       posNC=Isampling(suffdat,graph->CliquesDimens[i], graph->Cliques[i], 
       	       delta+SampleSize, tau, graph->Edge, nsamples*3/2);
       
       priNC=Isampling(NULL, graph->CliquesDimens[i], graph->Cliques[i], 
		       delta, tau, graph->Edge, nsamples/2);
              
       answer=answer+posNC-priNC;
              
     }
   }
else{
 answer= -100000.00;
 break;}
 }

for(i=0; i< graph->nSeparators; i++){
posNC=cliquelike(graph->Separators[i], graph->SeparatorsDimens[i], suffdat, tau, delta);
 answer=answer-posNC;
 

 }


return answer;

}



