#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <limits.h>
#include <iomanip>
#include <time.h>
extern int SampleSize;
extern int NumberOfGenes;
extern char DataFile[];
extern double **Data;

void CheckPointer(void* pointer)
{
   if(pointer==NULL)
   {
      printf("Memory allocation error.\n");
      exit(1);
   }
   return;
}

 
//computes n choose m
int choose(int m, int n)
{
   if(m > n) return 0;
   if(m == 0) return 1;
   if(m == 1) return n;
   if(m == n) return 1;
   if(m == (n-1)) return n;

   return(choose(m, n-1) + choose(m-1, n-1));
}

double logdexp(double x,double beta)
{
   return(-log(beta)-x/beta);
}

double logdpois(double x,double lambda)
{
   int i;
   double s = -lambda+x*log(lambda);

   for(i=2;i<=x;i++)
   {
      s += log(i);
   }

   return(s);
}

double getLogBetaPrior(double m,double x)
{
   double tau = 100;
   return((tau*m-1)*log(x)+(tau*(1-m)-1)*log(1-x));
}

double** ReadData()
{
   int i,j;
   double s;
   double** data;

   data = new double*[SampleSize];
   for(i=0;i<SampleSize;i++)
   {
      data[i] = new double[NumberOfGenes];
   }

   FILE* in = fopen(DataFile,"r");
   if(NULL==in)
   {
      printf("Cannot open data file %s.\n",DataFile);
      exit(1);
   }
   for(i=0;i<SampleSize;i++)
   {
      for(j=0;j<NumberOfGenes;j++)
      {
	 fscanf(in,"%lf",&s);
         data[i][j] = s;
      }
   }
   fclose(in);
   return data; 
}

void FreeData(double** Data)
{
   int i;

   for(i=0;i<SampleSize;i++)
   {
      delete[] Data[i]; 
      Data[i] = NULL;
   }
   delete[] Data;
   Data = NULL;
   return;
}

void NormalizeData(double** data)
{
   int i,j;
   double s;

   for(j=0;j<NumberOfGenes;j++)
   {
      s = 0.0;
      for(i=0;i<SampleSize;i++)
      {
	 s += data[i][j];
      }
      s = s/((double)SampleSize);
      for(i=0;i<SampleSize;i++)
      {
	 data[i][j] -= s;
      }
   }
   return;
}

double getSSD(double** data,int vi,int vj)
{
   int k;
   double s = 0.0;

   for(k=0;k<SampleSize;k++)
   {
      s += data[k][vi]*data[k][vj];     
   }
   return s;
}

double mylgamma(double lambda,int p)
{
   int i;
   double s = 0.0;

   for(i=0;i<p;i++)
   {
      s += lgamma(lambda-(((double)i)/2.0));
   }
   return(s);
}

